{*******************************************************}
{                                                       }
{       xTool - Component Collection                    }
{                                                       }
{       Copyright (c) 1995 Stefan Bther                }
{                                                       }
{*******************************************************}
{
   21.04.96  Added Locate method for querys               Stefc
   04.06.96  Added TListField                             Stefc
   04.08.96  Added GetNextNo function                     Stefc
   16.08.96  Added CopyRecord function                    Stefc
   16.08.96  Added CloneTable function                    Stefc
   19.08.96  Added ExecuteSQL procedure                   Stefc
   12.09.96  Added CheckTable from km                     Stefc
   12.09.96  Added SaveDictScript                         Stefc
   13.09.96  Some bugfixes in CheckTable                  Stefc
   29.09.96  Added LoadFromSQL                            Stefc
   07.02.97  Added TableDefsFromIni                       Stefc
   28.02.97  ParamCheck := False in PrepareQuery          Stefc
   21.03.97  BCD wird jetzt von TableDefsFromIni erkannt  Stefc
   04.04.97  TableDefsFromIni verkraftet fehlende Lngen  Stefc
   14.04.97  New methods HideFields & NameFields          Stefc
   25.04.97  Better support for Paradox local SQL         Stefc
   29.04.97  No CheckIndex any longer                     Stefc
   04.06.97  Support more datetypes in CheckTables        Stefc
   18.06.97  Now support Time datatype in CheckTables     Stefc
   04.08.97  make back compatible with Delphi 1.0         Stefc
   13.08.97  CheckTable can now reorganize tables         Stefc
   05.10.97  use virtual instead of ovveride (Bug D1.00)  Stefc
   08.10.98  DriverToSqlTyp also use INTERBASE as keyword Stefc
   12.10.98  Not use GetAliasType( any longer             Stefc
   04.01.99  New class TDictionaryUtils added             Stefc
   11.08.99  added xProcReg to uses clause                Stefc
   16.08.99  Warnings beseitigt                           ar
   13.11.99  TDictionaryUtils.commentLine neu             Stefc
}
unit xDbProcs;

interface

uses
  {$IFNDEF Win32} DbiProcs, DbiTypes, {$ELSE} Bde, {$ENDIF} WinProcs,
  Classes, SysUtils,
  xProcs, {$IFDEF Win32} xProcReg, {$ENDIF}
  Db, DbTables, Forms, IniFiles;

type
  { access gif fields within databases }
  TIncSearch = record
    LastTime : Integer;
    Search   : String;
    Elapsed  : Integer;
  end;

  TSqlTyp = (sqlUnknown,
             sqlDBase, sqlParadox, sqlAccess,
             sqlInterbase, sqlOracle, sqlSQLServer, sqlDB2 );

const
  LocalSQLs : set of TSQLTyp = [sqlDBase..sqlAccess];

type
  TListField = class(TStringField)
  private
    FValues  : TStringList;
    procedure   SetValues(Value: TStrings);
    function    GetValues: TStrings;
  protected
    procedure GetText(var Text: string; DisplayText: Boolean); override;
    procedure SetText(const Value: string); override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Values: TStrings read GetValues write SetValues;
  end;

  TIntListField = class(TIntegerField)
  private
    FValues  : TStringList;
    procedure   SetValues(Value: TStrings);
    function    GetValues: TStrings;
  protected
    procedure GetText(var Text: string; DisplayText: Boolean); override;
    procedure SetText(const Value: string); override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Values: TStrings read GetValues write SetValues;
  end;

  TDecimalField = class(TFloatField)
  private
    oldDecimal: Char;
    procedure Pre;
    procedure Post;
  protected
    function  GetAsFloat: Double; override;
    function  GetAsString: string; override;
    procedure GetText(var Text: string; DisplayText: Boolean); override;
    procedure SetAsFloat(Value: Double); override;
    procedure SetAsString(const Value: string); override;
  public
    constructor Create(aOwner: TComponent); override;
    function IsValidChar(Ch: Char): Boolean; override;
  end;

  TBook = class
  private
    FBookmark : TBookmark;
    FIndex    : String;
    FTable    : TTable;
  public
    constructor Create(aTable: TTable);
    constructor CreateIndexName(aTable: TTable; const aIndex: String);
    destructor  Destroy; override;
  end;

procedure CopyRecord(aSrc,aDest: TDataSet; aExclude: TStringList);
function  CloneTable(aTable: TTable): TTable;
procedure IncrementSearch(aDataSet: TDataSet; Key: Char; var aIncSearch: TIncSearch);
function  QueryLocate(aQuery: TQuery; aField, aValue: String): Boolean;
function  GetNextNo(aTable: TTable; const aField: String): Integer;
function  GetDataPath(const aAlias: String): String;
function  GetIndexValue(aTable: TTable): String;
procedure ExecuteSQL(const aAlias: String; const aSQL: array of String);
function  CalcSQL(const aAlias: String; const aSQL: array of String): String;
function  CountSQL(const aAlias: String; const aSQL: array of String): Integer;
procedure LoadFromSQL(const aAlias: String; const aSQL: array of String; List: TStrings);
procedure DeleteSQL(const aAlias, aTable, aWhere: String; aId: Integer);
procedure CheckTable(aSql: TSqlTyp; aTable: TTable; aFelder: TFielddefs; aInd: TIndexdefs;
                     aReorg: Boolean);
procedure TableDefsFromIni(aSql: TSqlTyp; const aFile, aTableName: String;
                           aFields : TFieldDefs; aIndex  : TIndexDefs);
procedure ConnectDataBase(aIni: TIniFile; aDb: TDataBase);
function  DriverToSqlTyp(const aDriver: String): TSQLTyp;
function  GetSqlTyp(aDb: TDataBase): TSqlTyp;
function  TempTableName: String;
procedure HideFields(DataSet: TDataSet; aFlds: array of String);
procedure NameFields(DataSet: TDataSet; aNames: TStrings);
procedure SetParadoxMasterKeyword(aDb: TDataBase; aTables : array of String;
                                  const aPassword: String);

procedure Register;

type
  TDictionaryUtils = class
  private
    class function StrToEnglishFloat(const S: String): Extended;
  public
    class function  commentLine( const S: String): String;
    class procedure addStrDefToFields(const aDef: String; aFields: TFieldDefs);
    class procedure addStrDefToIndexes(const aDef: String; aIndexes: TIndexDefs; aSql: TSqlTyp);
    class function  fieldTypeToStr(aType: TFieldType): String;
    class function  FieldToStrDef(aField: TFieldDef): String;
    class function  IndexToStrDef(aIndex: TIndexDef): String;
    class procedure generateDDScript(aTable: TTable; aOutput: TStrings);
    class procedure generatePasScript(aTable: TTable; aOutput: TStrings);
  end;


var
  AppUser : String[5];

implementation

uses
  TypInfo;

{ TDictionaryUtils }

class function TDictionaryUtils.StrToEnglishFloat(const S: String): Extended;
var
  oldSep: Char;
begin
  oldSep:=DecimalSeparator;
  DecimalSeparator:='.';
  Result:=StrToFloat(S);
  DecimalSeparator:=oldSep;
end;

class procedure TDictionaryUtils.addStrDefToFields(const aDef: String; aFields: TFieldDefs);
var
  S,aName,aTyp: String;
  aLen: Integer;
  aFloat: Extended;
  aRequire: Boolean;
begin
  S:=strTrim(commentLine(aDef));
  aName:=strTrim(strToken(S,'='));
  S:=strTrim(S);
  aTyp:= strToken(S,',');
  try
    aFloat:=StrToEnglishFloat(strToken(S,','));
    aLen:=Trunc(aFloat);
  except
    aLen := 0;                          {// 4.4.97 sb }
  end;
  aRequire:=strEqual(strToken(S,','),'True');
  with aFields do
    if strEqual(aTyp,'Char') or strEqual(aTyp,'VChar') then   {// String }
       Add(aName,ftString,aLen,aRequire)
    else if strEqual(aTyp,'Longint') then      {// 32-Bit }
       Add(aName,ftInteger,0,aRequire)
    else if strEqual(aTyp,'Number') then       {// 16-Bit }
       Add(aName,ftSmallInt,0,aRequire)
    else if strEqual(aTyp,'Float') then        {// Numerisch  }
       Add(aName,ftFloat,0,aRequire)           {// 3.4.97 sb  }
    else if strEqual(aTyp,'Bcd') then          {// Numerisch  }
       Add(aName,ftBCD,aLen,aRequire)
    else if strEqual(aTyp,'Bool') then         {// Boolean      }
       Add(aName,ftBoolean,0,aRequire)
    else if strEqual(aTyp,'Date') then         {// Datum        }
       Add(aName,ftDate,0,aRequire)
    else if strEqual(aTyp,'TimeStamp') or strEqual(aTyp,'DateTime') then    {// Datum & Zeit }
       Add(aName,ftDateTime,0,aRequire)
    else if strEqual(aTyp,'Time') then         {// Zeit }
       Add(aName,ftTime,0,aRequire)
    else if strEqual(aTyp,'Memo') then         { Memo }
       Add(aName,ftMemo,0,False)
    else if strEqual(aTyp,'Blob') then         { Blob }
       Add(aName,ftBlob,0,False)
    else
       raise Exception.Create(Format('unknown format "%s" in "TDictionaryUtils"',[aTyp]));
end;

class procedure TDictionaryUtils.addStrDefToIndexes(const aDef: String; aIndexes: TIndexDefs; aSql: TSqlTyp);
var
  S: String;
  aName,
  aFlds,
  aOpt : String;
  aOpts: TIndexOptions;
begin
  S:=strTrim(aDef);
  aName:=strTrim(strToken(S,'='));
  S:=strTrim(S);
  aFlds:= strToken(S,',');
  aOpts:=[];
  aOpt:=strToken(S,',');
  while aOpt<>'' do
  begin
    if strEqual(aOpt,'ixPrimary') then
       Include(aOpts,ixPrimary)
    else if strEqual(aOpt,'ixUnique' ) then
       Include(aOpts,ixUnique)
    else if strEqual(aOpt,'ixDescending' ) then
       Include(aOpts,ixDescending)
    else if strEqual(aOpt,'ixCaseInsensitive' ) then
       Include(aOpts,ixCaseInsensitive)
    else
       raise Exception.Create(Format('unknown indexoption "%s" in "CheckTable"',[aOpt]));
    aOpt:=strToken(S,',');
  end;

  case aSql of
    sqlParadox :   {// Sekundary-Indexes must be case insenstive }
      if not (ixPrimary in aOpts) then
         Include(aOpts,ixCaseInsensitive);

    sqlAccess  :   {// Access indexes are all case insensitive }
      Include(aOpts, ixCaseInsensitive);
  end;

  aIndexes.Add(aName,aFlds,aOpts);
end;

class function TDictionaryUtils.FieldTypeToStr(aType: TFieldType): String;
begin
  case aType of
    ftString      : Result:= 'Char';
    ftInteger     : Result:= 'Longint';
    ftDate        : Result:= 'Date';
    ftTime        : Result:= 'Time';
    ftDateTime    : Result:= 'TimeStamp';
    ftBoolean     : Result:= 'Boolean';
    ftMemo        : Result:= 'Memo';
    ftBlob        : Result:= 'Blob';
    ftFloat       : Result:= 'Float';
    ftSmallint    : Result:= 'Number';
    ftWord        : Result:= 'Word';
    ftCurrency    : Result:= 'Currency';
    ftBCD         : Result:= 'Bcd';
    ftBytes       : Result:= 'Bytes';
    ftVarBytes    : Result:= 'VarBytes';
    ftGraphic     : Result:= 'Graphic';
   {$IFDEF Win32}
    ftAutoInc     : Result:= 'AutoInc';
    ftFmtMemo     : Result:= 'Memo';
    ftParadoxOle  : Result:= 'Blob(POle)';
    ftDBaseOle    : Result:= 'Blob(DOle)';
    ftTypedBinary : Result:= 'Typed Binary';
   {$ENDIF}
    else            Result:= 'unknown';
  end;
end;

class function TDictionaryUtils.FieldToStrDef(aField: TFieldDef): String;
begin
  Result:=aField.Name + '=' +
          FieldTypeToStr(aField.DataType) + ',' +
          IntToStr(aField.Size) + ',' +
          BoolToStr(aField.Required);
end;

class function TDictionaryUtils.IndexToStrDef(aIndex: TIndexDef): String;
begin
  if aIndex.Name='' then
     Result:= 'ixMain'
  else
     Result:=aIndex.Name;
  Result:=Result + '=' + aIndex.Fields;
  if ixPrimary         in aIndex.Options then Result:=Result+',ixPrimary';
  if ixUnique          in aIndex.Options then Result:=Result+',ixUnique';
  if ixDescending      in aIndex.Options then Result:=Result+',ixDescending';
  if ixCaseInsensitive in aIndex.Options then Result:=Result+',ixCaseInsensitive';
end;

class function TDictionaryUtils.commentLine( const S: String): String;
var
  p: Integer;
begin
  Result:=S;
  if Length(S)> 0 then
  begin
    if S[1] = ';' then
       Result:=''
    else begin
      p:=Pos(' //', Result);
      if p>0 then
         Delete(Result,p,Length(Result));
      Result:=Trim(Result);
    end;
  end;
end;

class procedure TDictionaryUtils.generateDDScript(aTable: TTable; aOutput: TStrings);
var
  i: Integer;
  aName: String;
begin
  aTable.FieldDefs.Update;
  aTable.IndexDefs.Update;
  aName:=strUpper(ExtractName(aTable.TableName));
  with aOutput do
  begin
    Add(Format('[%s]', [aName]));
    for i:=0 to aTable.FieldDefs.Count-1 do
      Add(FieldToStrDef(aTable.FieldDefs[i]));
    Add('');
    if aTable.IndexDefs.Count > 0 then
    begin
      Add(Format('[%s_IDX]', [aName]));
      for i:=0 to aTable.IndexDefs.Count-1 do
        Add(IndexToStrDef(aTable.IndexDefs[i]));
    end;
    Add('');
  end;
end;

class procedure TDictionaryUtils.generatePasScript(aTable: TTable; aOutput: TStrings);

  function fldString(aFld: TFieldDef): String;
  begin
    Result:= ''''+aFld.Name+''',';
    Result:= Result+GetEnumName(TypeInfo(TFieldType),Ord(aFld.DataType))
      {$IFNDEF Win32}^{$ENDIF}+','+IntToStr(aFld.Size)+','+'False';
  end;

  function idxString(aIdx: TIndexDef): String;
  var
    S : String;
  begin
    Result:=''''+aIdx.Name+''','''+aIdx.Fields+''',[';
    S:='';
    if ixPrimary in aIdx.Options then S:=S+',ixPrimary';
    if ixUnique  in aIdx.Options then  S:=S+',ixUnique';
    if ixDescending  in aIdx.Options then  S:=S+',ixDescending';
    if ixCaseInsensitive in aIdx.Options then S:=S+',ixCaseInsensitive';
    Delete(S,1,1);
    Result:=Result+S+']';
  end;

var
  j     : Integer;
begin
  aTable.Fielddefs.Update;
  aTable.IndexDefs.Update;
  with aOutput do begin
    Add('{ '+ExtractFileName(aTable.TableName)+' }');
    Add('with aFields do begin');
    Add('  Clear;');
    for j:=0 to aTable.FieldDefs.Count-1 do
      Add('  Add('+fldString(aTable.FieldDefs.Items[j])+');');
    Add('end;');
    Add('with aIndex do begin');
    Add('  Clear;');
    for j:=0 to aTable.IndexDefs.Count-1 do
      Add('  Add('+idxString(aTable.IndexDefs.Items[j])+');');
    Add('end;');
    Add('');
  end;
end;

{ duplicate record from one dataset to the other }
procedure CopyRecord(aSrc,aDest: TDataSet; aExclude: TStringList);
var
  i,j : Integer;

  aField,
  aDstField : TField;
begin
  aDest.Insert;  { at first we need a new record }
  for i:=0 to aSrc.FieldCount-1 do
    try
      aField := aSrc.Fields[i];

      { Check if this field should be transfered }
      if Assigned(aExclude) and (aExclude.Find(aField.FieldName,j)) then
         Continue;

      { Check if I found a field with the same name for this }
      aDstField := aDest.FieldByName(aField.FieldName);
      if Assigned(aDstField) then
      begin

        { blobs and equal field types should be converted 1:1 }
        if aField is TBlobField then
           aDstField.Assign(aField)
        else if aField.DataType = aDstField.DataType then
           aDstField.Assign(aField)
        else
           aDstField.AsString := aField.AsString;  { do some type conversion }
      end;

    except
      { some error on transfer occur }
    end;
end;

{ Duplicate Table }
function DuplicateTable(aTable: TTable): TTable;
var
  aSrc: TTable;
begin
  Result := nil;
  try
    Result := TTable.create(nil);
    with Result do begin
      Databasename := aTable.DataBaseName;
      Tablename    := TempTableName;
      aSrc:=TTable.Create(nil);
      try
        with aSrc do
        begin
          DataBaseName:=aTable.DataBaseName;
          TableName:=aTable.TableName;
        end;
        aSrc.Open;
        BatchMove(aSrc,batCopy);
        aSrc.Close;
      finally
        aSrc.Free;
      end;
      Open;
    end;
  except
    Result.Free;
    raise;
  end;
end;

{ deliver a second open table }
function CloneTable(aTable: TTable): TTable;
begin
  Result:=TTable.Create(nil);
  with Result do
  begin
    DataBaseName:=aTable.DataBaseName;
    TableName:=aTable.TableName;
    IndexFieldNames:=aTable.IndexFieldNames;
    Open;
    GotoCurrent(aTable);
  end;
end;

function QueryLocate(aQuery: TQuery; aField, aValue: String): Boolean;
var
  Hi, Lo: Integer;
begin
  { - Left : + Find search field from ORDER BY clause
             + Bookmarking for FindKey emulation
  }

  with aQuery do
  begin
    First;
    Hi := RecordCount;
    Lo := 0;
    { Move to point half way between high and low ends of range }
    MoveBy(RecordCount div 2);
    while (Hi - Lo) > 1 do
    begin
      { Search field greater than search value, value in first half }
      if (FieldByName(AField).AsString > aValue) then
      begin
        { Lower high end of range by half of total range }
        Hi := Hi - ((Hi - Lo) div 2);
        MoveBy(((Hi - Lo) div 2) * -1);
      end else
      { Search field less than search value, value in far half }
      begin
        { Raise low end of range by half of total range }
        Lo := Lo + ((Hi - Lo) div 2);
        MoveBy((Hi - Lo) div 2);
      end;
    end; { while }
    { Fudge for odd numbered rows }
    if (FieldByName(AField).AsString > AValue) then Prior;

    Result:= (FieldByName(AField).AsString = AValue)
  end; { with }
end;

procedure IncrementSearch(aDataSet: TDataSet; Key: Char; var aIncSearch: TIncSearch);
var
  z      : Integer;
begin
  with aIncSearch do
  begin
    { reset after three seconds }
    z:=TimeToInt(Time);
    if (z-LastTime) > Elapsed then Search:='';
    LastTime:=z;

    if (Key=BACKSPACE) or ((Key > SPACE) and (Key <> chr(21))) then   { what is chr(21) ??? }
    begin
      if Key = BACKSPACE then
         Search := strLeft(Search,Length(Search)-1)
      else
         Search:=Search+Key;

      if aDataSet is TTable then
         with aDataSet as TTable do
           FindNearest([Search])
      else if aDataSet is TQuery then
         with aDataSet as TQuery do              { assumes first field is ORDER BY field !!!! }
{$IFDEF Win32}
           if Search = '' then
              Filter:=''
           else begin
              Filter:=Format('%s=''%s*''', [Fields[0].FieldName,Search]);
              FindFirst;
           end;
{$ELSE}
           QueryLocate(TQuery(aDataSet), Fields[0].AsString, Search);
{$ENDIF}
    end else
      Search := '';
  end;
end;

{ Retrieve next number after last record }
function GetNextNo(aTable: TTable; const aField: String): Integer;
var
  Table: TTable;
begin
  Result:=1;
  Table:=TTable.Create(nil);
  try
    with Table do
    begin
      DataBaseName:=aTable.DataBaseName;
      TableName:=aTable.TableName;
      ReadOnly:=True;
      IndexFieldNames:=aField;
      Open;
      Last;
      if eof then Result := FieldByName(aField).AsInteger+1;
    end;
  finally
    Table.Free;
  end;
end;

function GetDataPath(const aAlias: String): String;
var
  aParams : TStringList;
begin
  Result:='';
  aParams:=TStringList.Create;
  try
    Session.GetAliasParams(aAlias, aParams);
    Result:= aParams.Values['PATH'];
  finally
    aParams.Free;
  end;
end;

function GetIndexValue(aTable: TTable): String;
var
  i:Integer;
begin
  Result:='';
  for i:=0 to aTable.IndexFieldCount-1 do
    Result:=Result+';'+aTable.IndexFields[i].AsString;
  Delete(Result,1,1);
end;

function  TempTableName: String;
begin
  Result:='TMP'+IntToStr(Random(9999));
end;


function PrepareQuery(const aAlias: String; const aSQL: array of String): TQuery;
var
  i     : Integer;
begin
  Result:=TQuery.Create(nil);
  Result.DataBaseName := aAlias;
 {$IFDEF Win32}
  Result.ParamCheck:= False;         { 28.2.97 sb }
 {$ENDIF}
  for i:=Low(aSQL) to High(aSQL) do Result.SQL.Add(aSQL[i]);
end;

{ executes a SQL statement }
procedure ExecuteSQL(const aAlias: String; const aSQL: array of String);
begin
  with PrepareQuery(aAlias, aSQL) do
  try
    ExecSQL;
  finally
    Free;
  end;
end;

function CalcSQL(const aAlias: String; const aSQL: array of String): String;
begin
  Result:='';
  with PrepareQuery(aAlias, aSQL) do
  try
    Open;
    if not eof then
       Result:=Fields[0].AsString;
  finally
    Free;
  end;
end;

function CountSQL(const aAlias: String; const aSQL: array of String): Integer;
begin
  Result:=0;
  with PrepareQuery(aAlias, aSQL) do
  try
    Open;
    if not eof then
       Result:=Fields[0].AsInteger;
  finally
    Free;
  end;
end;

procedure LoadFromSQL(const aAlias: String; const aSQL: array of String; List: TStrings);
begin
  with PrepareQuery(aAlias, aSQL) do
  try
    List.Clear;
    Open;
    while not eof do
    begin
      List.Add(Fields[0].AsString);
      Next;
    end;
  finally
    Free;
  end;
end;

procedure DeleteSQL(const aAlias, aTable, aWhere: String; aId: Integer);
begin
  ExecuteSQL(aAlias,['DELETE FROM '+aTable+' WHERE ',Format(aWhere,[aId])]);
end;

{ Check if table exists and have the correct structure }
procedure CheckTable(aSql    : TSqlTyp;
                     aTable  : TTable;
                     aFelder : TFielddefs;
                     aInd    : TIndexdefs;
                     aReorg  : Boolean);

  { Prfen ob vorhanden, wenn nicht anlegen }
  function CheckExists: Boolean;
  begin
    Result:=True;
    try
      aTable.Fielddefs.Update;
    except
      on E: EDbEngineError do
        if (10024 <> E.Errors[0].ErrorCode) and
           (8712  <> E.Errors[0].ErrorCode) then
          Check(E.Errors[0].ErrorCode)
        else
        begin
          aTable.FieldDefs.Assign(aFelder);
          aTable.Indexdefs.Assign(aInd);
          aTable.CreateTable;
          Result := False;
        end;
      else
        raise;
    end;
  end;

  { Prfen ob Indexe bereinstimmen, evtl. ndern }
(*
  function CheckIndex: Boolean;
  var
    i : Integer;
    aIdx: TIndexDef;

    function CheckIdx(a,b:TIndexDef): Boolean;
    var
      aName: String;
    begin
      Result := strEqual(a.Fields,b.Fields);
      case aSql of
        sqlParadox, sqlAccess,
        sqlDBase     : Result := Result and
          strEqual(a.Name,b.Name) and (a.Options=b.Options);

        sqlInterbase : Result := Result and
          ((ixPrimary in a.Options) or strEqual(a.Name,b.Name)) and
          (a.Options=b.Options);

        sqlOracle    :
          begin
            aName:=a.Name;
            if strTokenCount(aName,'.')>0 then strToken(aName,'.');
            Result := Result and strEqual(aName,b.Name) and
               ((a.Options - [ixPrimary]) = b.Options)
          end;
      end;
    end;

  begin
    Result:=True;
    aTable.IndexDefs.Update;
    if aInd.Count <> aTable.IndexDefs.Count then
       Result:=False
    else for i:= 0 to aInd.Count-1 do
     try
       Application.ProcessMessages;
       aIdx:=aTable.IndexDefs.Items[i];
       if not CheckIdx(aIdx,aInd.Items[i]) then
       begin
         Result:=False;
         break;
       end;
     except
       Result:=False;
       break;
     end;
  end;
*)

  { Prfen ob Felder bereinstimmen }
  function CheckFields: Boolean;
  var
    i   : Integer;
    aFld: TFieldDef;

    function CheckTyp(a,b:TFieldDef): Boolean;
    begin
      Result := (a.DataType = b.DataType) and (a.Size = b.Size);
      case aSql of
        sqlOracle    : Result := Result or
          (a.DataType = ftFloat) and (b.DataType=ftInteger) or
          (a.DataType = ftString) and (b.DataType=ftBoolean) or
          (a.DataType = ftDate) and (b.DataType=ftTime);
      end;
    end;

  begin
    Result:=True;
    if aFelder.Count <> aTable.FieldDefs.Count then
       Result:=False
    else for i:=0 to aFelder.Count-1 do
      try
        Application.ProcessMessages;
        aFld:=aTable.Fielddefs.Find(aFelder.Items[i].Name);
        if not ((aFld<> nil ) and CheckTyp(aFld,aFelder.Items[i])) then
        begin
          Result:=False;
          break;
        end;
      except
         on E: EDbEngineError do
           begin
             Result:=False;
             break;
           end
         else
           raise;
      end;
  end;

  procedure ConvertTable;
  var
    aNew,aCopy : TTable;
    aFld       : TField;
    i          : Integer;
  begin
    aCopy := DuplicateTable(aTable);
    try
      aNew := TTable.Create(nil);
      try
        with aNew do
        begin
          { define }
          DataBaseName:=aTable.DataBaseName;
          TableName:=aTable.TableName;
          TableType:=aTable.TableType;

          { create }
          FieldDefs.Assign(aFelder);
          IndexDefs.Assign(aInd);
          CreateTable;

          { copy old records }
          with TBatchMove.Create(nil) do
          try
            Destination := aNew;
            Source := aCopy;
            Mode := batAppend;
            for i:=0 to aFelder.Count-1 do
            begin
              aFld:=aCopy.FindField(aFelder.Items[i].Name);
              if aFld<>nil then
                 Mappings.Add(aFelder.Items[i].Name+'='+aFelder.Items[i].Name);
            end;
            Execute;
          finally
            Free;
          end;
        end; {with}
      finally
        aNew.Free;
      end;

      aCopy.Close;
      aCopy.DeleteTable;

    finally
      aCopy.Free;
    end;
  end;

begin
  try
    if CheckExists then
       if not CheckFields or aReorg { or not CheckIndex } then
          ConvertTable;
  except
    on E : Exception do
    begin
      Information(Format('Problems with table "%s" !'#13'%s',[aTable.TableName,E.Message]));
      raise;
    end;
  end;
end;


procedure ConnectDataBase(aIni: TIniFile; aDb: TDataBase);
const
  _Alias = 'Alias';
  _Db    = 'Db';
  _Type  = 'Type';
  _Pass  = 'PASSWORD';
  _User  = 'USER NAME';
var
  aList  : TStringList;
  aAlias : String;
begin
  with aIni do
  begin
    aAlias:= ReadString(_Db, _Alias, '');

    aList := TStringList.Create;
    try
      ReadSection(ReadString(_Db,_Type,''),aList);

      { create alias to our database }
{$IFDEF Win32}  { only possible under Delphi 2.0 ! }
      with Session do
        if not IsAlias(aAlias) then
        begin
          ReadSection(ReadString(_Db,_Type,''),aList);
          aList.Values[_User]:=ReadString(_Db,_User, '');
          aList.Values[_Pass]:=ReadString(_Db,_Pass, '');
          AddAlias(aAlias, ReadString(_Db,_Type,''), aList);
        end;
{$ENDIF}

      { login in database }
      with aDb do
      begin
        Connected    := False;
        AliasName    := aAlias;
        DataBaseName := 'WORK';
        Params.Assign(aList);
        Params.Values[_User]:=ReadString(_Db,_User,'');
        Params.Values[_Pass]:=ReadString(_Db,_Pass,'');
        Connected := True;
      end;

    finally
      aList.Free;
    end;
  end;
end;

(*
function GetAliasType(const AliasName: String): String;
var
  SAlias: array[0..31] of Char;
  Desc  : DBDesc;
begin
  StrPLCopy(SAlias, AliasName, SizeOf(SAlias) - 1);
  AnsiToOem(SAlias, SAlias);
  Check(DbiGetDatabaseDesc(SAlias, @Desc));
  OemToAnsi(Desc.szDbType,Desc.szDbType);
  Result:=Format('%s',[Desc.szDbType]);
end;
*)

function DriverToSqlTyp(const aDriver: String): TSQLTyp;
begin
  Result:=sqlUnknown;
  if strEqual(aDriver,'ORACLE') then
     Result:=sqlOracle
  else if strEqual(aDriver,'INTRBASE') or strEqual(aDriver,'INTERBASE') then
     Result:=sqlInterbase
  else if strEqual(aDriver,'DB2') then
     Result:=sqlDb2
  else if strEqual(aDriver,'MSACCESS') then
     Result:= sqlAccess;
end;

function  GetSqlTyp(aDb: TDataBase): TSqlTyp;
var
  S: String;
begin
  Result:=DriverToSqlTyp(aDb.DriverName);
  if Result=sqlUnknown then
  begin
    S:=aDb.Params.Values['DEFAULT DRIVER'];
    if strEqual(S,'PARADOX') then
       Result := sqlParadox
    else if strEqual(S,'DBASE') then
       Result:= sqlDBase;
  end;
end;

procedure TableDefsFromIni(aSql: TSqlTyp; const aFile, aTableName: String;
                           aFields : TFieldDefs; aIndex  : TIndexDefs);

  function StrToEnglishFloat(const S: String): Extended;
  var
    oldSep: Char;
  begin
    oldSep:=DecimalSeparator;
    DecimalSeparator:='.';
    Result:=StrToFloat(S);
    DecimalSeparator:=oldSep;
  end;

var
  aList : TStringList;
  i     : Integer;
 (* aTyp,
  aName,
  aFlds, *)
 (*
  aFloat   : Float;
  aDigits,
  aLen     : Integer;
  aRequire : Boolean; *)
begin
  aList:=TStringList.Create;
  try
    with TIniFile.Create(aFile) do
    try
      {// Felder einlesen und aufbauen }
      with aFields do begin
        Clear;
        ReadSectionValues(aTableName,aList);
        for i:=0 to aList.Count-1 do
        begin
          TdictionaryUtils.addStrDefToFields(aList[i], aFields);
          (*
          S       := ReadString(aTableName,aList[i],'');
          aName   := aList[i];
          aTyp    := strToken(S,',');
          try
            aFloat:=StrToEnglishFloat(strToken(S,','));
            aLen:=Trunc(aFloat);
            aDigits:=Trunc((aFloat-Trunc(aFloat))*10);
          except
            aLen := 0;                          {// 4.4.97 sb }
          end;
          aRequire:=strEqual(strToken(S,','),'True');
          if strEqual(aTyp,'Char') or strEqual(aTyp,'VChar') then   {// String }
             Add(aName,ftString,aLen,aRequire)
          else if strEqual(aTyp,'Longint') then      {// 32-Bit }
             Add(aName,ftInteger,0,aRequire)
          else if strEqual(aTyp,'Number') then       {// 16-Bit }
             Add(aName,ftSmallInt,0,aRequire)
          else if strEqual(aTyp,'Float') then        {// Numerisch  }
             Add(aName,ftFloat,0,aRequire)           {// 3.4.97 sb  }
          else if strEqual(aTyp,'Bcd') then          {// Numerisch  }
             Add(aName,ftBCD,aLen,aRequire)
          else if strEqual(aTyp,'Bool') then         {// Boolean      }
             Add(aName,ftBoolean,0,aRequire)
          else if strEqual(aTyp,'Date') then         {// Datum        }
             Add(aName,ftDate,0,aRequire)
          else if strEqual(aTyp,'TimeStamp') then    {// Datum & Zeit }
             Add(aName,ftDateTime,0,aRequire)
          else if strEqual(aTyp,'Time') then         {// Zeit }
             Add(aName,ftTime,0,aRequire)
          else if strEqual(aTyp,'Memo') then         { Memo }
             Add(aName,ftMemo,0,False)
          else if strEqual(aTyp,'Blob') then         { Blob }
             Add(aName,ftBlob,0,False)
          else
             raise Exception.Create(Format('unknown format "%s" in "CheckTable"',[aTyp]));
             *)
        end;
      end;

      {// Index einlesen und aufbauen }
      with aIndex do begin
        Clear;
        aList.Clear;
        ReadSectionValues(aTableName+'_IDX',aList);
        for i:=0 to aList.Count-1 do
        begin
          TDictionaryUtils.addStrDefToIndexes(aList[i], aIndex, aSql);

         (*
          S:=ReadString(aTableName+'_IDX',aList[i],'');
          aName:=aList[i];
          aFlds:=strToken(S,',');
          aOpts:=[];
          aOpt:=strToken(S,',');
          while aOpt<>'' do
          begin
            if strEqual(aOpt,'ixPrimary') then
               Include(aOpts,ixPrimary)
            else if strEqual(aOpt,'ixUnique' ) then
               Include(aOpts,ixUnique)
            else if strEqual(aOpt,'ixDescending' ) then
               Include(aOpts,ixDescending)
            else if strEqual(aOpt,'ixCaseInsensitive' ) then
               Include(aOpts,ixCaseInsensitive)
            else
               raise Exception.Create(Format('unknown indexoption "%s" in "CheckTable"',[aOpt]));
            aOpt:=strToken(S,',');
          end;

          case aSql of
            sqlParadox :   {// Sekundary-Indexes must be case insenstive }
              if not (ixPrimary in aOpts) then
                 Include(aOpts,ixCaseInsensitive);

            sqlAccess  :   {// Access indexes are all case insensitive }
              Include(aOpts, ixCaseInsensitive);
          end;

          Add(aName,aFlds,aOpts);
          *)

        end;
      end;

    finally
      Free;
    end;
  finally
    aList.Free;
  end;
end;

procedure HideFields(DataSet: TDataSet; aFlds: array of String);
var
  i: Integer;
begin
  for i:=Low(aFlds) to High(aFlds) do
    DataSet.FieldByName(aFlds[i]).Visible:=False;
end;

procedure NameFields(DataSet: TDataSet; aNames: TStrings);
var
 {$IFNDEF Win32}
  p,
 {$ENDIF}
  i: Integer;
  aName: String;
begin
  for i:=0 to aNames.Count-1 do
  begin
{$IFDEF Win32}
    aName:=aNames.Names[i];
{$ELSE}
    aName:=aNames[i];
    P:=Pos('=',aName);
    if P<>0 then
       aName:=strLeft(aName,P-1)
    else
       continue;
{$ENDIF}
    DataSet.FieldByName(aName).DisplayLabel:=aNames.Values[aName];
  end;
end;

{ TListField }

constructor TListField.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FValues:=TStringList.Create;
end;

destructor TListField.Destroy;
begin
  FValues.Free;
  inherited Destroy;
end;

procedure TListField.SetValues(Value: TStrings);
begin
  FValues.Assign(Value);
end;

function TListField.GetValues: TStrings;
begin
  Result:=FValues;
end;

procedure TListField.GetText(var Text: string; DisplayText: Boolean);
begin
  inherited GetText(Text,DisplayText);
  if FValues.Count > 0 then
     Text := FValues.Values[AnsiUpperCase(Text)];
end;

procedure TListField.SetText(const Value: string);
var
  S,S1 : String;
  i,p  : Integer;
begin
  S1:=Value;
  if FValues.Count > 0 then
    for i:=0 to FValues.Count-1 do
    begin
      S:=FValues[i];
      p:=Pos('=',S);
      if P>0 then
        if strEqual(Value, strRight(S,Length(S)-P)) then
        begin
          S1:=strLeft(S,P-1);
          break;
        end;
    end;
  inherited SetText(S1);
end;

{ TIntListField }

constructor TIntListField.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FValues:=TStringList.Create;
end;

destructor TIntListField.Destroy;
begin
  FValues.Free;
  inherited Destroy;
end;

procedure TIntListField.SetValues(Value: TStrings);
begin
  FValues.Assign(Value);
end;

function TIntListField.GetValues: TStrings;
begin
  Result:=FValues;
end;

procedure TIntListField.GetText(var Text: string; DisplayText: Boolean);
begin
  inherited GetText(Text,DisplayText);
  if FValues.Count > 0 then
     Text := FValues[StrToInt(Text)];
end;

procedure TIntListField.SetText(const Value: string);
var
  i : Integer;
begin
  i:=FValues.IndexOf(Value);  {// 6.1.96 sb  Find funktioniert nur bei sortierten Listen ! }
  if i<>-1 then
    inherited SetText(IntToStr(i))
  else
    inherited SetText(Value);
end;

{ TDecimalField }

constructor TDecimalField.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  oldDecimal:=DecimalSeparator;
end;

function TDecimalField.GetAsFloat: Double;
begin
  Pre;
  Result := inherited GetAsFloat;
  Post;
end;

function TDecimalField.GetAsString: string;
begin
  Pre;
  Result := inherited GetAsString;
  Post;
end;

procedure TDecimalField.GetText(var Text: string; DisplayText: Boolean);
begin
  Pre;
  inherited GetText(Text,DisplayText);
  Post;
end;

function TDecimalField.IsValidChar(Ch: Char): Boolean;
begin
  Pre;
  Result := inherited IsValidChar(Ch);
  Post;
end;

procedure TDecimalField.SetAsFloat(Value: Double);
begin
  Pre;
  inherited SetAsFloat(Value);
  Post;
end;

procedure TDecimalField.SetAsString(const Value: string);
begin
  Pre;
  inherited SetAsString(Value);
  Post;
end;

procedure TDecimalField.Pre;
begin
  DecimalSeparator:='.';
end;

procedure TDecimalField.Post;
begin
  DecimalSeparator:=oldDecimal;
end;

{ TBook }

constructor TBook.Create(aTable: TTable);
begin
  FTable:=aTable;
  FBookmark:=aTable.GetBookmark;
end;

constructor TBook.CreateIndexName(aTable: TTable; const aIndex: String);
begin
  FTable:=aTable;
  FIndex:=FTable.IndexName;
  FTable.IndexName:=aIndex;
end;

destructor TBook.Destroy;
begin
  if FIndex<>'' then
     FTable.IndexName := FIndex
  else begin
    FTable.GotoBookmark(FBookmark);
    FTable.FreeBookmark(FBookmark);
  end;
  inherited Destroy;
end;

{ Registration }

procedure Register;
begin
  RegisterFields([TListField,TIntListField,TDecimalField]);
end;

procedure InitUser;
var
  S: String;
begin
 {$IFDEF Win32}
  AppUser:='';
  S:=TRegUtils.CurrentUser;            // 11.08.99 sb
  if Length(S)<=3 then
    AppUser:=S
  else
    repeat
      AppUser:=AppUser+strLeft(strToken(S,' '),1);
    until S='';
  AppUser:=strLower(AppUser);
 {$ELSE}
  AppUser:='xx';
 {$ENDIF}
end;

procedure SetParadoxMasterKeyword(aDb: TDataBase; aTables : array of String;
                                  const aPassword: String);
var
  aInfo : CRTblDesc;
  i     : Integer;
begin
  for i:=Low(aTables) to High(aTables) do
  begin
    Fillchar(aInfo,Sizeof(aInfo),0);
    with aInfo do begin
      StrPCopy(szTblName,ChangeFileExt(aTables[i],'.db'));
      StrPCopy(szPassword,aPassword);
      bProtected:=True;
      bPack:=True;
    end;
    Check(DbiDoRestructure (              { Restructure a table }
        aDb.Handle,
        1,                                { Number of table descriptors (1) }
        @aInfo,                           { Array of table descs }
        nil,                              { Restructure to this table }
        nil,                              { Keyviol table name  (optional) }
        nil,                              { Problems table name (optional) }
        false                             { Analyze restructure }
     ));
  end;
end;

begin
  InitUser;
end.
