unit xProcDateTime;
  { date & time methods }

interface

uses
  SysUtils;

const
  NULL_DATE     : TDateTime = 0.0;

type
  TMonth        = (NoneMonth,January,February,March,April,May,June,July,
                   August,September,October,November,December);

  TDayOfWeek    = (Sunday,Monday,Tuesday,Wednesday,Thursday,Friday,Saturday);

  { date functions }
  TDateUtils = class
    class function  Year(D: TDateTime): Integer;
    class function  Month(D: TDateTime): Integer;
    class function  Day(D: TDateTime): Integer;
    class function  BeginOfYear(D: TDateTime): TDateTime;
    class function  EndOfYear(D: TDateTime): TDateTime;
    class function  BeginOfMonth(D: TDateTime): TDateTime;
    class function  EndOfMonth(D: TDateTime): TDateTime;
    class function  WeekOfYear(D: TDateTime): Integer;
    class function  DayOfYear(D: TDateTime): Integer;
    class function  DayOfWeek(D: TDateTime): TDayOfWeek;
    class function  LeapYear(D: TDateTime): Boolean;
    class function  BeginOfQuarter(D: TDateTime): TDateTime;
    class function  EndOfQuarter(D: TDateTime): TDateTime;
    class function  BeginOfWeek(D: TDateTime;Weekday: Integer): TDateTime;
    class function  DaysInMonth(D: TDateTime): Integer;
    class function  Quicken(D: TDateTime; var Key: Char): TDateTime;
  end;

  { time functions }
  TTimeUtils = class
    class function  Hour(T: TDateTime): Integer;
    class function  Min(T: TDateTime): Integer;
    class function  Sec(T: TDateTime): Integer;
    class function  ToInt(T: TDateTime): Integer;
  end;

implementation

{ date calculations }

class function  TDateUtils.Year(D: TDateTime): Integer;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=Year;
end;

class function  TDateUtils.Month(D: TDateTime): Integer;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=Month;
end;

class function TDateUtils.BeginOfYear(D: TDateTime): TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=EncodeDate(Year,1,1);
end;

class function  TDateUtils.EndOfYear(D: TDateTime): TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=EncodeDate(Year,12,31);
end;

class function TDateUtils.BeginOfMonth(D: TDateTime): TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=EncodeDate(Year,Month,1);
end;

class function TDateUtils.EndOfMonth(D: TDateTime): TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  if Month=12 then
  begin
    Inc(Year);
    Month:=1;
  end else
    Inc(Month);
  Result:=EncodeDate(Year,Month,1)-1;
end;

class function TDateUtils.WeekOfYear(D: TDateTime): Integer; { Armin Hanisch }
const
  t1: array[1..7] of ShortInt = ( -1,  0,  1,  2,  3, -3, -2);
  t2: array[1..7] of ShortInt = ( -4,  2,  1,  0, -1, -2, -3);
var
  doy1,
  doy2    : Integer;
  NewYear : TDateTime;
begin
  NewYear:=BeginOfYear(D);
  doy1 := DayofYear(D) + t1[SysUtils.DayOfWeek(NewYear)];
  doy2 := DayofYear(D) + t2[SysUtils.DayOfWeek(D)];
  if doy1 <= 0 then
    Result := WeekOfYear(NewYear-1)
  else if (doy2 >= DayofYear(EndOfYear(NewYear))) then
    Result:= 1
  else
    Result:=(doy1-1) div 7+1;
end;

class function TDateUtils.DayOfYear(D: TDateTime): Integer;
begin
  Result:=Trunc(D-BeginOfYear(D))+1;
end;

class function TDateUtils.DayOfWeek(D: TDateTime): TDayOfWeek;
begin
  Result:=TDayOfWeek(Pred(SysUtils.DayOfWeek(D)));
end;

class function TDateUtils.LeapYear(D: TDateTime): Boolean;
var
  Year,Month,Day: Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=(Year mod 4 = 0) and ((Year mod 100 <> 0) or (Year mod 400 = 0));
end;

class function TDateUtils.BeginOfQuarter(D: TDateTime):TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=EncodeDate(Year,(((Month-1) div 3) * 3)+1,1);   { sb 11.02.97 }
end;

class function TDateUtils.EndOfQuarter(D: TDateTime): TDateTime;
begin
  Result:=BeginOfQuarter(BeginOfQuarter(D)+(3*31))-1;
end;

class function TDateUtils.BeginOfWeek(D: TDateTime; Weekday: Integer): TDateTime;
begin
  Result:=D;
  while SysUtils.DayOfWeek(Result)<>Weekday do Result:=Result-1;
end;

class function TDateUtils.DaysInMonth(D: TDateTime): Integer;
const
  DaysPerMonth: array[1..12] of Byte= (31,28,31,30,31,30,31,31,30,31,30,31);
var
  aMonth: Integer;
begin
  aMonth:=Month(D);
  Result:=DaysPerMonth[aMonth];
  if (aMonth=2) and LeapYear(D) then Inc(Result);
end;

class function TDateUtils.Day(D: TDateTime): Integer;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=Day;
end;

class function TDateUtils.Quicken(D: TDateTime; var Key: Char): TDateTime;
const
 {$IFDEF German}
  _ToDay    = 'H';
  _PrevYear = 'J';
  _NextYear = 'R';
  _PrevMonth= 'M';
  _NextMonth= 'T';
  _BeginQuart='Q';
  _EndQuart  ='U';
 {$ELSE}
  _ToDay    = 'T';
  _PrevYear = 'Y';
  _NextYear = 'R';
  _PrevMonth= 'M';
  _NextMonth= 'H';
  _BeginQuart='Q';
  _EndQuart  ='U';
 {$ENDIF}
begin
  case Upcase(Key) of                     { Quicken Date Fast Keys }
    '+'        : Result := D+1;
    '-'        : Result := D-1;
    _ToDay     : Result := Date;
    _PrevYear  : if D <> BeginOfYear(D)  then Result:=BeginOfYear(D)
                                         else Result:=BeginOfYear(D-1);
    _NextYear  : if D <> EndOfYear(D)    then Result:=EndOfYear(D)
                                         else Result:=EndOfYear(Date+1);
    _PrevMonth : if D <> BeginOfMonth(D) then Result:=BeginOfMonth(D)
                                         else Result:=BeginOfMonth(D-1);
    _NextMonth : if D <> EndOfMonth(D)   then Result:=EndOfMonth(D)
                                         else Result:=EndOfMonth(D+1);
    _BeginQuart: Result := BeginOfQuarter(D);
    _EndQuart  : Result := EndOfQuarter(D);
    else begin
      Result := D;
      exit;
    end;
  end;
  Key:=#0;
end;

{ time functions }

class function TTimeUtils.Hour(T: TDateTime): Integer;
var
  Hour,Minute,Sec,Sec100: Word;
begin
  DecodeTime(T,Hour,Minute,Sec,Sec100);
  Result:=Hour;
end;

class function TTimeUtils.Min(T: TDateTime): Integer;
var
  Hour,Minute,Sec,Sec100: Word;
begin
  DecodeTime(T,Hour,Minute,Sec,Sec100);
  Result:=Minute;
end;

class function  TTimeUtils.Sec(T: TDateTime): Integer;
var
  Hour,Minute,Sec,Sec100: Word;
begin
  DecodeTime(T,Hour,Minute,Sec,Sec100);
  Result:=Sec;
end;

class function TTimeUtils.ToInt(T: TDateTime): Integer;
begin
  Result:=Trunc((MSecsPerday * T) / 1000);
end;


end.
