unit xProcSys;
{-------------------------------------------------------------------------------
 System functions

 Historie:

   11.05.99  sb  TempPath function vergessen
   27.07.99  sb  Methode zum Bestimmen ob wir in der Delphi32-IDE sind.
   05.08.99  ar  Abhngigkeit von Forms.pas bei Konsolenanwendungen aufgehoben.

 -------------------------------------------------------------------------------}

interface

uses
 {$IFNDEF Win32}
  WinTypes, WinProcs;
 {$ELSE}
  Windows;
 {$ENDIF}

type

{$IFNDEF Win32}
  DWORD        = Longint;
{$ENDIF}

  TScreenColor = ( scUnknown, scMonoChrome, sc16Color, sc256Color,
                   scHiColor, scTrueColor );

  TSysUtils = class
  public
    class function  TempPath: String;
    class procedure Delay(aMs: DWORD);
    class procedure HardDelay(aMs: DWORD);
    class procedure Beep;
    class function  ColorDepth: Integer;
    class function  ScreenColor: TScreenColor;
   {$IFDEF Win32}
    class function  ComputerName: String;
    class function  UserName: String;
    class function  ModuleName: String;
    class function  EnvVariable(const aName: String): String;
   {$ENDIF}
    class function  isInIDE: Boolean;
  end;

  { easy access of the system metrics }
  TSystemMetric = class
  private
    FMenuHeight,
    FCaptionHeight : Integer;
    FBorder,
    FFrame,
    FDlgFrame,
    FBitmap,
    FHScroll,
    FVScroll,
    FThumb,
    FFullScreen,
    FMin,
    FMinTrack,
    FCursor,
    FIcon,
    FDoubleClick,
    FIconSpacing : TPoint;
  protected
    constructor Create;
    procedure Update;
    class function getSystemPoint(ax,ay: Integer):TPoint;
  public
    property MenuHeight: Integer read FMenuHeight;
    property CaptionHeight: Integer read FCaptionHeight;
    property Border: TPoint read FBorder;
    property Frame: TPoint read FFrame;
    property DlgFrame: TPoint read FDlgFrame;
    property Bitmap: TPoint read FBitmap;
    property HScroll: TPoint read FHScroll;
    property VScroll: TPoint read FVScroll;
    property Thumb: TPoint read FThumb;
    property FullScreen: TPoint read FFullScreen;
    property Min: TPoint read FMin;
    property MinTrack: TPoint read FMinTrack;
    property Cursor: TPoint read FCursor;
    property Ico: TPoint read FIcon;
    property DoubleClick: TPoint read FDoubleClick;
    property IconSpacing: TPoint read FIconSpacing;
  end;

var
  SysMetric: TSystemMetric;

implementation

uses
  SysUtils, Classes {$IFDEF CONSOLE}, Messages{$ELSE}, Forms{$ENDIF};

{ TSysUtils }

{===============================================================================}
{$IFDEF CONSOLE}
function ProcessMessage(var Msg: TMsg): Boolean;
begin
  Result := False;
  if PeekMessage(Msg, 0, 0, 0, PM_REMOVE) then
  begin
    Result := True;
    if Msg.Message <> WM_QUIT then
    begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end
    else
      Halt;
  end;
end;

procedure ProcessMessages;
var
  Msg: TMsg;
begin
  while ProcessMessage(Msg) do {loop};
end;
{$ENDIF}

{===============================================================================}
class procedure TSysUtils.Delay(aMs: DWORD);
var
  TickCount: DWORD;
begin
  TickCount := GetTickCount;
  while GetTickCount - TickCount < aMs do
   {$IFDEF CONSOLE}
    ProcessMessages
   {$ELSE}
    Application.ProcessMessages
   {$ENDIF};
end;

class procedure TSysUtils.HardDelay(aMs: DWORD);
var
  TickCount: DWORD;
begin
  TickCount := GetTickCount;
  while GetTickCount - TickCount < aMs do;
end;

class procedure TSysUtils.Beep;
begin
  MessageBeep($FFFF);
end;

class function TSysUtils.ColorDepth: Integer;
var
  aDC: hDC;
begin
  aDC := GetDC(0);
  try
    Result := 1 shl (GetDeviceCaps(aDC,PLANES) * GetDeviceCaps(aDC, BITSPIXEL));
  finally
    ReleaseDC(0, aDC);
  end;
end;

class function TSysUtils.ScreenColor: TScreenColor;    { 13.08.97 }
var
  aDC : hDC;
begin
  Result:= scUnknown;
  aDC := GetDC( 0 );
  try
    case GetDeviceCaps( aDC, BITSPIXEL ) of
      1 : Result := scMonoChrome;
      4	: Result := sc16Color;
      8	: Result := sc256Color;
     15,
     16	: Result := scHiColor;
     24,
     32	: Result := scTrueColor;
    end;
  finally
    ReleaseDC( 0, aDC);
  end;
end;

{$IFDEF Win32}
class function TSysUtils.TempPath: String;
var
  Buffer: array[0..1023] of Char;
begin
  SetString(Result, Buffer, GetTempPath(Sizeof(Buffer)-1,Buffer));
end;
{$ELSE}
class function TSysUtils.TempPath:String;
var
  Buffer: array[0..255] of char;
begin
  GetTempFileName(#0,'TMP',0,Buffer);             { 15.07.96 sb }
  Result:=StrPas(Buffer);
  DeleteFile(Result);
  Result:=ExtractFilePath(Result);
end;
{$ENDIF}


{$IFDEF Win32 }
{// Ermittelt den Computer-Namen }
class function TSysUtils.ComputerName: String;
var
  Buffer : array[0..MAX_COMPUTERNAME_LENGTH] of Char;
  aLen   : dWord;
begin
  aLen:=Sizeof(Buffer);
  if GetComputerName(Buffer,aLen) then
     SetString(Result, Buffer, aLen)
  else
     Result:='';
end;

class function TSysUtils.UserName: String;
var
  Buffer: array[0..100] of char;
  aLen: dWord;
begin
  aLen:=Sizeof(Buffer);
  if GetUserName(Buffer,aLen) then
     SetString(Result, Buffer, aLen)
  else
     Result:='';
end;

class function TSysUtils.ModuleName: String;
var
  Buffer: array[0..255] of Char;
begin
  GetModuleFileName(HInstance, Buffer, SizeOf(Buffer));
  OemToAnsi(Buffer, Buffer);
  Result:=StrPas(Buffer);
end;

class function TSysUtils.EnvVariable(const aName: String): String;
var
  Buffer: array[0..255] of char;
begin
  if GetEnvironmentVariable(PChar(aName),Buffer,Sizeof(Buffer))<>0 then
     Result:=StrPas(Buffer)
  else
     Result:='';
end;
{$ENDIF}

class function TSysUtils.isInIDE: Boolean;
var
  aContainer: String;
begin
  aContainer := ChangeFileExt(ExtractFileName(Paramstr(0)), '');
  Result:=AnsiCompareText(aContainer,'delphi32')=0;
end;

{ TSystemMetric }

constructor TSystemMetric.Create;
begin
  inherited Create;
  Update;
end;

class function TSystemMetric.getSystemPoint(ax,ay: Integer):TPoint;
begin
  Result:=Point(GetSystemMetrics(ax),GetSystemMetrics(ay));
end;

procedure TSystemMetric.Update;
begin
  FMenuHeight    :=GetSystemMetrics(SM_CYMENU);
  FCaptionHeight :=GetSystemMetrics(SM_CYCAPTION);
  FBorder        :=GetSystemPoint(SM_CXBORDER,SM_CYBORDER);
  FFrame         :=GetSystemPoint(SM_CXFRAME,SM_CYFRAME);
  FDlgFrame      :=GetSystemPoint(SM_CXDLGFRAME,SM_CYDLGFRAME);
  FBitmap        :=GetSystemPoint(SM_CXSIZE,SM_CYSIZE);
  FHScroll       :=GetSystemPoint(SM_CXHSCROLL,SM_CYHSCROLL);
  FVScroll       :=GetSystemPoint(SM_CXVSCROLL,SM_CYVSCROLL);
  FThumb         :=GetSystemPoint(SM_CXHTHUMB,SM_CYVTHUMB);
  FFullScreen    :=GetSystemPoint(SM_CXFULLSCREEN,SM_CYFULLSCREEN);
  FMin           :=GetSystemPoint(SM_CXMIN,SM_CYMIN);
  FMinTrack      :=GetSystemPoint(SM_CXMINTRACK,SM_CYMINTRACK);
  FCursor        :=GetSystemPoint(SM_CXCURSOR,SM_CYCURSOR);
  FIcon          :=GetSystemPoint(SM_CXICON,SM_CYICON);
  FDoubleClick   :=GetSystemPoint(SM_CXDOUBLECLK,SM_CYDOUBLECLK);
  FIconSpacing   :=GetSystemPoint(SM_CXICONSPACING,SM_CYICONSPACING);
end;

{$IFNDEF Win32}
procedure donexProcSys; far;
begin
  SysMetric.Free;
end;
{$ENDIF}

initialization

  SysMetric:= TSystemMetric.Create;

{$IFDEF Win32}
finalization
  SysMetric.Free;
{$ELSE}
  AddExitProc(DoneXProcSys);
{$ENDIF}
end.
