unit xTextLog;
  { dumps some text with timestamp to a textfile }

{ 23.03.99 sb Immer anlegen
  26.03.99 sb div.
  11.05.99 sb Unit xProcSys verwendet
  31.05.99 sb Weitere Mglichkeiten des Loggings eingebaut
  03.06.99 sb Untersttzung des SQL-Monitors /sqlmon eingebaut
  03.06.99 sb Console-Window bentigt kein #13 !
  17.06.99 sb log=<file> wird nun untersttzt
  05.08.99 ar Abhngigkeit von Forms.pas bei Konsolenanwendungen aufgehoben.    !!!!!!!!!!!!!!!!!!!
}

interface

uses
  Classes;

type
  TTextLog = class(TPersistent)
  private
    fActive  : Boolean;
  protected
    fNoStamp : Boolean;
    procedure activate; virtual;
    procedure _dump(const aText: String); virtual; abstract;
  public
    procedure dump(const aText: String);
  end;

var
  Log : TTextLog = nil;

implementation

uses
  Windows, Messages, SysUtils,
  xProcs, xProcSys;

type
  { TNullTextLog }
  TNullTextLog = class(TTextLog)
  protected
    procedure _dump(const aText: String); override;
  end;

  { TFileTextLog }
  TFileTextLog = class(TTextLog)
  private
    fFile : String;
  protected
    procedure _dump(const aText: String); override;
  public
    constructor Create(const aFile: String);
    property FileName: String read fFile write fFile;
  end;

  { TConsoleTextLog }
  TConsoleTextLog = class(TTextLog)
  protected
    procedure _dump(const aText: String); override;
  end;

  { TDualTextLog }
  TDualTextLog = class(TTextLog)
  private
    fFileLog,
    fConsoleLog : TTextLog;
  protected
    procedure _dump(const aText: String); override;
  public
    constructor Create(const aFile: String);
    destructor Destroy; override;
  end;


{ TTextLog }

procedure TTextLog.activate;
begin
  fActive:=True;
end;

procedure TTextLog.Dump(const aText: String);

  function getTime: String;
  var
    aTime: TSystemTime;
  begin
    GetLocalTime(aTime);
    with aTime do
      Result := Format( '%2d:%2d:%2d:%3d', [wHour, wMinute, wSecond, wMilliSeconds]);
  end;

begin
  if not fActive then
     activate;
  if fNoStamp then
     _dump( aText )
  else
     _dump( getTime {DateTimeToStr(Now)} + #9 + aText );
end;

{ TFileTextLog }
constructor TFileTextLog.Create(const aFile: String);
begin
  inherited Create;
  fFile:=aFile;
end;

procedure TFileTextLog._Dump(const aText: String);
var
  f: Textfile;
begin
  AssignFile(f,fFile);
  if fileSize(fFile) > 1000 * 1024 then
    Rewrite(f)
  else
   try
     Append(f);
   except
     Rewrite(f);
   end;
  writeln(f, aText );
  CloseFile(f);
end;

{ TNullTextLog }
procedure TNullTextLog._dump(const aText: String);
begin end;

{ TConsoleTextLog }
procedure TConsoleTextLog._dump(const aText: String);
var
  P    : PChar;
  CPS  : TCopyDataStruct;
  aWnd : hWnd;
begin
  aWnd := FindWindow('TfrmDbgTerm', nil);
  if aWnd <> 0 then
  begin
    CPS.cbData := Length(aText) + 1;
    GetMem(P, CPS.cbData);
    try
      StrPCopy(P, aText);
      CPS.lpData := P;
      SendMessage(aWnd, WM_COPYDATA, 0, LParam(@CPS));
    finally
      FreeMem(P, Length(aText)+1);
    end;
  end;
end;

{ TDualTextLog }
constructor TDualTextLog.Create(const aFile: String);
begin
  inherited Create;
  fFileLog := TFileTextLog.Create(aFile);
  fConsoleLog := TConsoleTextLog.Create;
end;

destructor TDualTextLog.Destroy;
begin
  fFileLog.Free;
  fConsoleLog.Free;
  inherited Destroy;
end;

procedure TDualTextLog._dump(const aText: String);
begin
  if fFileLog<>nil then fFileLog._dump(aText);
  if fConsoleLog<>nil then fConsoleLog._dump(aText);
end;

function checkParam(const aName: String): Boolean;
var
  aLine,aToken: String;
begin
  Result:=False;
  aLine:=strCmdLine;
  aToken:=strToken(aLine,';');
  while aToken<>'' do
  begin
    if AnsiCompareText(aToken,aName)=0 then
    begin
      Result:=True;
      break;
    end;
    aToken:=strToken(aLine,';');
  end;
end;

function checkFile(const aName: String): String;
var
  aLine,aToken,aItem,aValue: String;
begin
  Result:='';
  aLine:=strCmdLine;
  aToken:=strToken(aLine,';');
  while aToken<>'' do
  begin
    if strSplitItemValue(aToken,aItem,aValue) then
      if AnsiCompareText(aItem,aName)=0 then
      begin
        Result:=aValue;
        break;
      end;
    aToken:=strToken(aLine,';');
  end;
end;

var
  aFile: String;

initialization

  aFile := checkFile('log');
  if aFile <> '' then
     Log := TFileTextLog.Create( aFile )
  else if checkParam('log') then
     Log := TFileTextLog.Create(TSysUtils.TempPath + ExtractName(ParamStr(0)) + '.log')
  else if checkParam('dual') then
     Log := TDualTextLog.Create(TSysUtils.TempPath + ExtractName(ParamStr(0)) + '.log')
  else if checkParam( 'con') then
     Log := TConsoleTextLog.Create
  else
     Log := TNullTextLog.Create;

finalization

  Log.Free;

end.
