unit xUIDlg;
{-------------------------------------------------------------------------------
 User-Interface Dialogs

 SOLL DIALOGS-/FORM-frei bleiben! Nur Standard-Windows-Routinen verwenden!

 Historie:

   11.08.98  sb  Anlage
   11.08.99  ar  TWindowsUIDialogs verffentlicht. MsgBox berarbeitet. Prozedur
                 'Warning' implementiert. Globale Funktionen aus xProcs
                 jetzt hier implementiert.
 -------------------------------------------------------------------------------}

interface

type
  TUIDialogs = class
  public
    function  Question(const aMsg: String):Boolean; virtual; abstract;
    procedure Information(const aMsg: String); virtual; abstract;
    function  Confirmation(const aMsg: String): Word; virtual; abstract;
    procedure Warning(const aMsg: String); virtual; abstract;
    procedure Error(const aMsg: String); virtual; abstract;
  end;

  TWindowsUIDialogs = class(TUIDialogs)
  private
    function MsgBox(aTitle, aMsg: String; aFlag: Integer): Integer;
  public
    function  Question(const aMsg: String):Boolean; override;
    procedure Information(const aMsg: String); override;
    function  Confirmation(const aMsg: String): Word; override;
    procedure Warning(const aMsg: String); override;
    procedure Error(const aMsg: String); override;
  end;

{// -- public functions }
function  Question(const aMsg: String):Boolean;
procedure Information(const aMsg: String);
function  Confirmation(const aMsg: String): Word;
procedure Warning(const aMsg: String);
procedure Error(const aMsg: String);

var
  UIDialogs : TUIDialogs;

procedure SetUIDialogs(aValue: TUIDialogs);

implementation

{$i xProcs.inc}

uses
 {$IFDEF Win32}
  Windows,
 {$ELSE}
  WinTypes, WinProcs,
 {$ENDIF}
  SysUtils, Consts;

{$IFNDEF Win32}
const
  MB_ICONWARNING                 = MB_ICONEXCLAMATION;
  MB_ICONERROR                   = MB_ICONHAND;
  MB_ICONINFORMATION             = MB_ICONASTERISK;
  MB_ICONSTOP                    = MB_ICONHAND;
{$ENDIF}

function TWindowsUIDialogs.MsgBox(aTitle, aMsg: String; aFlag: Integer): Integer;
var
  p: Integer;
 {$IFNDEF Win32}
  cMsg,cTitle: array[0..255] of char;
 {$ENDIF}
begin
 {// -- Delphi 4 hat DT_NOPREFIX im DrawText der Nachricht vergessen, daher mu man
  // alle '&' doppelt angeben, wenn man diese ausgeben will. Strings mit '&&'
  // hier fr die Windowsfunktion auf '&' zurcksetzen. }
  p := Pos('&&', aMsg);
  while p > 0 do
  begin
    Delete(aMsg, p, 1);
    p := Pos('&&', aMsg);
  end;

  {// -- Standard Windows-Fenster ausgeben }
 {$IFDEF Win32}
  Result := MessageBox(GetActiveWindow, PChar(aMsg), PChar(aTitle), aFlag);
 {$ELSE}
  Result := MessageBox(GetActiveWindow, StrPLCopy(cMsg,aMsg,Sizeof(cMsg)-1),
                                        StrPLCopy(cTitle,aTitle,Sizeof(cTitle)-1),
                                        aFlag);
 {$ENDIF}
end;

function TWindowsUIDialogs.Question(const aMsg: String):Boolean;
begin
  Result:=MsgBox({$IFNDEF RESSTR}LoadStr{$ENDIF}(SMsgDlgConfirm),
                 aMsg, MB_ICONQUESTION or MB_YESNO) = IDYES
end;

procedure TWindowsUIDialogs.Information(const aMsg: String);
begin
  MsgBox({$IFNDEF RESSTR}LoadStr{$ENDIF}(SMsgDlgInformation),
         aMsg, MB_ICONINFORMATION or MB_OK )
end;

function TWindowsUIDialogs.Confirmation(const aMsg: String): Word;
begin
  case MsgBox({$IFNDEF RESSTR}LoadStr{$ENDIF}(SMsgDlgConfirm),
              aMsg,MB_ICONQUESTION or MB_YESNOCANCEL) of
   IDYES    : Result := idYes;
   IDNO     : Result := idNo;
   IDCANCEL : Result := idCancel;
   else       Result := idCancel;
  end;
end;

procedure TWindowsUIDialogs.Warning(const aMsg: String);
begin
  MsgBox({$IFNDEF RESSTR}LoadStr{$ENDIF}(SMsgDlgWarning),
         aMsg, MB_ICONWARNING or MB_OK);
end;

procedure TWindowsUIDialogs.Error(const aMsg: String);
begin
  MsgBox({$IFNDEF RESSTR}LoadStr{$ENDIF}(SMsgDlgError),
         aMsg, MB_ICONERROR or MB_OK);
end;

{=== Aus xProcs bernommene globale Nachrichtenfunktionen ======================}
function Question(const aMsg: String):Boolean;
begin
  if Assigned(UIDialogs) then
    Result := UIDialogs.Question(aMsg)
  else
    Result := False;
end;

procedure Information(const aMsg: String);
begin
  if Assigned(UIDialogs) then
    UIDialogs.Information(aMsg);
end;

function Confirmation(const aMsg: String): Word;
begin
  if Assigned(UIDialogs) then
    Result := UIDialogs.Confirmation(aMsg)
  else
    Result := idCancel;
end;

procedure Warning(const aMsg: String);
begin
  if Assigned(UIDialogs) then
    UIDialogs.Warning(aMsg);
end;

procedure Error(const aMsg: String);
begin
  if Assigned(UIDialogs) then
    UIDialogs.Error(aMsg);
end;

procedure SetUIDialogs(aValue: TUIDialogs);
begin
  if UIDialogs <> nil then
    UIDialogs.Free;
  UIDialogs := aValue;
end;

{$IFNDEF Win32}
procedure DoneXUIDlg; far;
begin
  SetUIDialogs(nil);
end;
{$ENDIF}


initialization

  UIDialogs := nil;
  SetUIDialogs(TWindowsUIDialogs.Create);

{$IFDEF Win32}
finalization
  SetUIDialogs(nil);
{$ELSE}
  AddExitProc(DoneXUIDlg);
{$ENDIF}

end.
